<?php

namespace Drupal\term_manager\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;

/**
 * Class AdminToolbarToolsSettingsForm.
 *
 * @package Drupal\admin_toolbar_tools\Form
 */
class TermManagerMoveForm extends ConfigFormBase {

  /**
   * The cache menu instance.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheMenu;

  /**
   * The menu link manager instance.
   *
   * @var \Drupal\Core\Menu\MenuLinkManagerInterface
   */
  protected $menuLinkManager;

  /**
   * AdminToolbarToolsSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menuLinkManager
   *   A menu link manager instance.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheMenu
   *   A cache menu instance.
   */
  public function __construct(ConfigFactoryInterface $configFactory, MenuLinkManagerInterface $menuLinkManager, CacheBackendInterface $cacheMenu) {
    parent::__construct($configFactory);
    $this->cacheMenu = $cacheMenu;
    $this->menuLinkManager = $menuLinkManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.menu.link'),
      $container->get('cache.menu')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'term_manager.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'term_manager_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('term_manager.settings');
    $term_depth = $config->get('term_manager_depth');
    
    $vocabularies = Vocabulary::loadMultiple();

    $voc_array= array();
    $term_array = array();
    
    foreach($vocabularies as $voc) {
      $value = $voc->label();
      $term_array[$value] = $value;
        
      for ($i=1; $i<= $term_depth; $i++) {
        $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($value, 0, $i);
        if (!is_null($terms)) {
          //print "<pre>"; print_r($terms); print "</pre>"; exit;    
          foreach ($terms as $term) {
            $term_array[$term->tid] = str_repeat("-", $term->depth + 1) . $term->name . " ($term->tid)";
          }    
        }
      }
    }
        
    $form['term_manager_tid'] = [
      '#type' => 'select',
      '#title' => $this->t('Source of tid'),
      '#default_value' => $config->get('term_manager_nid'),
      '#required' => FALSE,
      '#description' => $this->t('Select tid to move to new Vocabulary.'),
      '#options' => $term_array,
    ];
    
    
    foreach($vocabularies as $voc) {
      $value = $voc->label();
      $voc_array[$value] = $value;
    }
      
    $form['term_manager_vid'] = [
      '#type' => 'select',
      '#title' => $this->t('Target of vid'),
      '#default_value' => $config->get('term_manager_vid'),
      '#required' => FALSE,
      '#description' => $this->t('Select vid target.'),
      '#options' => $voc_array,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
      
    $term_manager_tid = $form_state->getValue('term_manager_tid');
    $term_manager_vid = $form_state->getValue('term_manager_vid');
    
    if (!is_numeric($term_manager_tid)) {
      \Drupal::messenger()->addWarning("Select tid first.");
      return;
    }
    
    $message = "Tid $term_manager_tid move to vid $term_manager_vid";
    \Drupal::messenger()->addStatus($message); 
    \Drupal::service('cache.render')->invalidateAll();
     
    $term = Term::load($term_manager_tid);
    $term->vid->setValue($term_manager_vid);
    $term->Save();
    
    
    // $url = Url::fromRoute('entity.node.canonical', ['node' => $node_protector_now]);
    // $redirect = new RedirectResponse($url->toString());
    // $redirect->send();
    
    // \Drupal::service('cache.render')->invalidateAll();
    $this->cacheMenu->invalidateAll();
    $this->menuLinkManager->rebuild();
    
  }

}
